/***********************************************************************************
*                 DAE Tools Project: www.daetools.com
*                 Copyright (C) Dragan Nikolic
************************************************************************************
DAE Tools is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License version 3 as published by the Free Software
Foundation. DAE Tools is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with the
DAE Tools software; if not, see <http://www.gnu.org/licenses/>.
***********************************************************************************/
#include "model.h"
#include "call_stats.h"
#include <stdlib.h>
#include <unistd.h>
using namespace daetools_model;

/* Run using:
    $ ./cxx-cs-benchmark Nresidual_runs Njacobian_runs
    $ ./cxx-cs-benchmark 1000 100
*/
int main(int argc, char *argv[])
{
    std::string inputDirectory = getcwd(NULL, 0);
    inputDirectory += "/input-data-120x96";
    int Nresidual_runs = 1;
    int Njacobian_runs = 1;

    if(argc == 3)
    {
        Nresidual_runs = atoi(argv[1]);
        Njacobian_runs = atoi(argv[2]);
    }
    else
    {
        printf("Usage: cxx-cs-benchmark Nresidual_runs Njacobian_runs\n");
        return -1;
    }

    printf("Input data directory = %s\n", inputDirectory.c_str());
    printf("Number of residuals runs = %d\n", Nresidual_runs);
    printf("Number of jacobian runs  = %d\n", Njacobian_runs);

    try
    {
        LoadModel(inputDirectory);

        call_stats::TimeAndCount res_cxx, res_cs, jac_cxx, jac_cs;

        for(int i = 0; i < Nresidual_runs; i++)
        {
            printf("\rEvaluateResiduals C++: %4d/%d", i+1, Nresidual_runs);
            fflush(stdout);

            call_stats::TimerCounter tc(res_cxx);
            EvaluateResiduals_cxx();
        }
        printf("\n");

        for(int i = 0; i < Nresidual_runs; i++)
        {
            printf("\rEvaluateResiduals  CS: %4d/%d", i+1, Nresidual_runs);
            fflush(stdout);

            call_stats::TimerCounter tc(res_cs);
            EvaluateResiduals_cs();
        }
        printf("\n");

        for(int i = 0; i < Njacobian_runs; i++)
        {
            printf("\rEvaluateJacobian  C++: %4d/%d", i+1, Njacobian_runs);
            fflush(stdout);

            call_stats::TimerCounter tc(jac_cxx);
            EvaluateJacobian_cxx();
        }
        printf("\n");

        for(int i = 0; i < Njacobian_runs; i++)
        {
            printf("\rEvaluateJacobian   CS: %4d/%d", i+1, Njacobian_runs);
            fflush(stdout);

            call_stats::TimerCounter tc(jac_cs);
            EvaluateJacobian_cs();
        }
        printf("\n");

        //CompareResiduals();
        //CompareJacobian();

        printf("Time for EvaluateResiduals C++ = %.10f\n", res_cxx.duration);
        printf("Time for EvaluateResiduals CS  = %.10f\n", res_cs.duration);
        printf("Time for EvaluateJacobian  C++ = %.10f\n", jac_cxx.duration);
        printf("Time for EvaluateJacobian  CS  = %.10f\n", jac_cs.duration);

        printf("Relative time for EvaluateResiduals CS/C++ = %.4f\n", res_cs.duration/res_cxx.duration);
        printf("Relative time for EvaluateJacobian  CS/C++ = %.4f\n", jac_cs.duration/jac_cxx.duration);
        FreeModel();
    }
    catch(std::exception& e)
    {
        std::cout << e.what() << std::endl;
    }

    return 0;
}


