/***********************************************************************************
*                 DAE Tools Project: www.daetools.com
*                 Copyright (C) Dragan Nikolic
************************************************************************************
DAE Tools is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License version 3 as published by the Free Software
Foundation. DAE Tools is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with the
DAE Tools software; if not, see <http://www.gnu.org/licenses/>.
***********************************************************************************/
#ifndef DAE_MPI_TYPEDEFS_H
#define DAE_MPI_TYPEDEFS_H

#include <iostream>
#include <string>
#include <vector>
#include <map>
#include <string.h>

#ifndef real_t
#define real_t double
#endif

namespace daetools_model
{
void LoadModel(const std::string& inputDirectory);
void FreeModel();
void EvaluateResiduals_cxx();
void EvaluateResiduals_cs();
void EvaluateJacobian_cxx();
void EvaluateJacobian_cs();
void CompareResiduals();
void CompareJacobian();

/* Runtime information data structure. */
typedef struct runtimeInformationData_
{
    uint32_t Ntotal_vars;
    uint32_t Nequations;
    uint32_t Nequations_local;
    uint32_t Ndofs;
    real_t   startTime;
    real_t   timeHorizon;
    real_t   reportingInterval;
    real_t   relativeTolerance;
    bool     quasiSteadyState;

    std::vector<real_t>      dofs;
    std::vector<real_t>      init_values;
    std::vector<real_t>      init_derivatives;
    std::vector<real_t>      absolute_tolerances;
    std::vector<int32_t>     ids;
    std::vector<std::string> variable_names;
} runtimeInformationData_t;
}


#endif
